//-----------------------------------------------------------------------
//      Generic array class
//      Copyright (C) MIKAMI, Naoki 2005
//-----------------------------------------------------------------------

#ifndef MK_MyArray

#include <cassert>      // for assert()
#include <new>          // for new, delete, and set_new_handler()

template <class T> class Array
{
private:
    T    *v;
    int  size;                                  // size of array
    void Range(int pos) const;                  // range checking for Array
    void Copy(const Array<T>& v_src);           // copy of object
    inline void ArrayNew(const int i);          // routine for constructor
public:
    explicit Array(int i = 1) { ArrayNew(i); }  // default constructor
    Array(const Array<T>& a) { Copy(a); }       // copy constructor
    inline Array(int i, T init_v);              // constructor with initialization
    ~Array() { delete[] v; }                    // destructor
    void SetSize(int i);                        // setting size
    int Length() const { return size; }         // get size of array
    Array<T>& operator=(const Array<T>& a);     // assignment
    inline T& operator[](int i);                // assignment by element
    inline const T& operator[](int i) const;    // get element
    operator const T* () const { return v; }    // type conversion
    operator T* () const { return v; }          // type conversion
};

//-----------------------------------------------------------------------
// implementation of generic array class
//-----------------------------------------------------------------------

template <class T> void Array<T>::Range(int pos) const
{
    assert(0 <= pos && pos < size);     // out of bounds ?
}

template <class T> void Array<T>::Copy(const Array<T>& v_src)
{
    v = new T[size = v_src.size];
    assert(v);                          // allocation error
    for (int i=0; i<size; i++) v[i] = v_src.v[i];
}

// routine for constructor
template <class T> void Array<T>::ArrayNew(int i)
{
    std::set_new_handler(0);
    assert(i > 0);                      // illegal size
    v = new T[size = i];
    assert(v);                          // allocation error
}

// constructor with initialization
template <class T> inline Array<T>::Array(int i, T init_v)
{
    ArrayNew(i);
    for (int n=0; n<i; n++) v[n] = init_v;
}

template <class T> void Array<T>::SetSize(int i)
{
    delete[] v;
    assert(i > 0);                      // illegal size
    v = new T[size = i];
    assert(v);                          // allocation error
}

template <class T> Array<T>& Array<T>::operator=(const Array<T>& a)
{
    if (this != &a)                     // prohibition of self-assignment
    {
        delete [] v;
        Copy(a);
    }
    return *this;
}

template <class T> inline T& Array<T>::operator[](int i)
{
#ifdef DEBUG_ARRAY_CHECK
    Range(i);                               // out of bound ?
#endif
    return v[i];
}

template <class T> inline const T& Array<T>::operator[](int i) const
{
#ifdef DEBUG_ARRAY_CHECK
    Range(i);                               // out of bounds ?
#endif
    return v[i];
}

#define MK_MyArray
#endif
